package net.seesaa.nekokiss.util.dungeon_generator.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.KeyboardFocusManager;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpinnerNumberModel;
import javax.swing.ToolTipManager;
import javax.swing.border.BevelBorder;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileNameExtensionFilter;

import net.seesaa.nekokiss.util.Xor128;
import net.seesaa.nekokiss.util.dungeon_generator.Map;

final class GraphicalFrame extends JFrame implements MouseListener,
		MouseMotionListener {
	
	private static final Xor128 RND = new Xor128();
	private static final StringBuilder SB = new StringBuilder(263);
	private final JPanel contentPane;
	private final JLabel messageLabel;
	private final JLabel coordinateLabel;
	private final GraphicalPanel graphicalPanel;
	private final JTextArea outputTextArea;
	private final JButton generateButton;
	private final JButton destroyButton;
	private final JButton regenerateButton;
	private final JButton saveImageButton;
	private final JSpinner widthSpinner, heightSpinner;
	private final JSpinner minRoomsSpinner, maxRoomsSpinner;
	private final JSpinner minRoomWidthSpinner, maxRoomWidthSpinner;
	private final JSpinner minRoomHeightSpinner, maxRoomHeightSpinner;
	private final JSpinner maxConnectLengthSpinner;
	private final JSpinner tinyRoomRatioSpinner, tinyPassageRatioSpinner;
	private final JSpinner intersectRatioSpinner;
	private final JSpinner minModPassagesSpinner, maxModPassagesSpinner;
	private final JSpinner permissibleFailsSpinner;
	private final JCheckBox autoSeedsCheckBox;
	private final JSpinner seedXSpinner, seedYSpinner;
	private final JSpinner seedZSpinner, seedWSpinner;
	private final JCheckBox animationCheckBox;
	private final JLabel drawRoomSpeedLabel;
	private final JSpinner drawRoomSpeedSpinner;
	private final JCheckBox fillRoomCheckBox;
	private final JLabel drawPassageSpeedLabel;
	private final JSpinner drawPassageSpeedSpinner;
	private final JSpinner roomRedSpinner;
	private final JSpinner roomGreenSpinner;
	private final JSpinner roomBlueSpinner;
	private final JSpinner passageRedSpinner;
	private final JSpinner passageGreenSpinner;
	private final JSpinner passageBlueSpinner;
	private final JSpinner doorwayRedSpinner;
	private final JSpinner doorwayGreenSpinner;
	private final JSpinner doorwayBlueSpinner;
	private final JFileChooser imageChooser;
	private boolean changedRoomColor = true;
	private boolean changedPassageColor = true;
	private boolean changedDoorwayColor = true;
	
	private GraphicalFrame() {
		super("Dungeon Generator");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		Font m15Font = new Font(Font.MONOSPACED, Font.PLAIN, 15);
		Font m18Font = new Font(Font.MONOSPACED, Font.PLAIN, 18);
		
		this.contentPane = new JPanel(new BorderLayout());
		this.contentPane.setBorder(new EmptyBorder(4, 4, 4, 4));
		this.setContentPane(this.contentPane);
		
		// 下部
		JPanel statusBar = new JPanel(new BorderLayout());
		statusBar.setBorder(new BevelBorder(BevelBorder.LOWERED));
		this.contentPane.add(statusBar, BorderLayout.SOUTH);
		
		this.messageLabel = new JLabel();
		this.messageLabel.setFont(m18Font);
		statusBar.add(this.messageLabel, BorderLayout.CENTER);
		
		this.coordinateLabel = new JLabel("X(XXXX), Y(XXXX)");
		this.coordinateLabel.setFont(m18Font);
		statusBar.add(this.coordinateLabel, BorderLayout.EAST);
		
		// 左側
		JPanel leftPanel = new JPanel(new BorderLayout());
		this.contentPane.add(leftPanel, BorderLayout.CENTER);
		
		// 左側中央
		this.graphicalPanel = new GraphicalPanel(this);
		this.graphicalPanel.addMouseListener(this);
		this.graphicalPanel.addMouseMotionListener(this);
		this.graphicalPanel.setPreferredSize(new Dimension(256, 256));
		
		JScrollPane grapScrPane = new JScrollPane(this.graphicalPanel);
		leftPanel.add(grapScrPane, BorderLayout.CENTER);
		
		// 左側下部
		this.outputTextArea = new JTextArea(5, 31);	// height, width
		this.outputTextArea.setBorder(new CompoundBorder(new EtchedBorder(
				EtchedBorder.LOWERED), new EmptyBorder(4, 8, 4, 8)));
		this.outputTextArea.setFont(m18Font);
		this.outputTextArea.setEditable(false);
		this.outputTextArea.setLineWrap(true);
		//this.outputTextArea.setBackground(UIManager.getColor("Panel
		//		.background"));
		//this.outputTextArea.setHighlighter(null);
		
		JScrollPane outputScrPane = new JScrollPane(this.outputTextArea,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		leftPanel.add(outputScrPane, BorderLayout.SOUTH);
		
		// 右側
		JPanel rightPanel = new JPanel(new BorderLayout());
		this.contentPane.add(rightPanel, BorderLayout.EAST);
		
		// 右側最上部
		GridBagLayout gbl = new GridBagLayout();
		
		JPanel rightTopPanel = new JPanel(gbl);
		rightTopPanel.setBorder(new EtchedBorder());
		rightPanel.add(rightTopPanel, BorderLayout.NORTH);

		GridBagConstraints gbc = new GridBagConstraints();
		
		Insets buttonInsets = new Insets(2, 2, 2, 2);
		
		this.generateButton = new JButton("生成");
		this.generateButton.addActionListener(new ActionListener() {
			@Override
            public final void actionPerformed(ActionEvent e) {
				KeyboardFocusManager.getCurrentKeyboardFocusManager()
						.clearGlobalFocusOwner();
				GraphicalFrame.this.generateButton.setEnabled(false);
				GraphicalFrame.this.graphicalPanel.setVisible(true);
				GraphicalFrame.this.generate();
            }
		});
		this.generateButton.setFont(m18Font);
		this.generateButton.setMargin(buttonInsets);
		//gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 0.25d;
		gbl.setConstraints(this.generateButton, gbc);
		rightTopPanel.add(this.generateButton);
		
		this.destroyButton = new JButton("破棄");
		this.destroyButton.setEnabled(false);
		this.destroyButton.addActionListener(new ActionListener() {
			@Override
            public final void actionPerformed(ActionEvent e) {
				KeyboardFocusManager.getCurrentKeyboardFocusManager()
						.clearGlobalFocusOwner();
				GraphicalFrame.this.destroyButton.setEnabled(false);
				GraphicalFrame.this.regenerateButton.setEnabled(false);
				GraphicalFrame.this.saveImageButton.setEnabled(false);
				GraphicalFrame.this.graphicalPanel.destroy();
				GraphicalFrame.this.generateButton.setEnabled(true);
			}
		});
		this.destroyButton.setFont(m18Font);
		this.destroyButton.setMargin(buttonInsets);
		++gbc.gridx;
		gbl.setConstraints(this.destroyButton, gbc);
		rightTopPanel.add(this.destroyButton);
		
		this.regenerateButton = new JButton("再生成");
		this.regenerateButton.setEnabled(false);
		this.regenerateButton.addActionListener(new ActionListener() {
			@Override
            public final void actionPerformed(ActionEvent e) {
				KeyboardFocusManager.getCurrentKeyboardFocusManager()
						.clearGlobalFocusOwner();
				GraphicalFrame.this.destroyButton.setEnabled(false);
				GraphicalFrame.this.regenerateButton.setEnabled(false);
				GraphicalFrame.this.saveImageButton.setEnabled(false);
				GraphicalFrame.this.generate();
			}
		});
		this.regenerateButton.setFont(m18Font);
		this.regenerateButton.setMargin(buttonInsets);
		++gbc.gridx;
		gbl.setConstraints(this.regenerateButton, gbc);
		rightTopPanel.add(this.regenerateButton);
		
		this.saveImageButton = new JButton("保存");
		this.saveImageButton.setEnabled(false);
		this.saveImageButton.addActionListener(new ActionListener() {
			@Override
            public final void actionPerformed(ActionEvent e) {
				KeyboardFocusManager.getCurrentKeyboardFocusManager()
						.clearGlobalFocusOwner();
				GraphicalFrame.this.saveImage();
			}
		});
		this.saveImageButton.setFont(m18Font);
		this.saveImageButton.setMargin(buttonInsets);
		++gbc.gridx;
		gbl.setConstraints(this.saveImageButton, gbc);
		rightTopPanel.add(this.saveImageButton);
		
		// 右側上部
		JPanel rightPanel2 = new JPanel(new BorderLayout());
		
		JScrollPane rightScrPane = new JScrollPane(rightPanel2,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		rightPanel.add(rightScrPane, BorderLayout.CENTER);
		
		gbl = new GridBagLayout();
		
		TitledBorder tb;
		
		JPanel settingPanel = new JPanel(gbl);
		tb = new TitledBorder(new EtchedBorder(), "生成条件設定");
		tb.setTitleFont(m15Font);
		settingPanel.setBorder(tb);
		rightPanel2.add(settingPanel, BorderLayout.NORTH);
		
		JLabel widthLabel = new JLabel("マップの幅");
		widthLabel.setFont(m18Font);
		widthLabel.setToolTipText("<html>引数名 : mapWidth<br>Min " +
				Map.MIN_MAP_WIDTH + "<br>Max " + Map.MAX_MAP_WIDTH + "</html>");
		gbc.anchor = GridBagConstraints.WEST;
		//gbc.fill = GridBagConstraints.NONE;
		gbc.gridx = 0;
		gbc.weightx = 0;
		gbl.setConstraints(widthLabel, gbc);
		settingPanel.add(widthLabel);
		
		this.widthSpinner = new JSpinner(new SpinnerNumberModel(75,
				Map.MIN_MAP_WIDTH, Map.MAX_MAP_WIDTH, 5));
		this.widthSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.widthSpinner, gbc);
		settingPanel.add(this.widthSpinner);
		
		JLabel heightLabel = new JLabel("マップの高さ");
		heightLabel.setFont(m18Font);
		heightLabel.setToolTipText("<html>引数名 : mapHeight<br>Min " + Map
				.MIN_MAP_HEIGHT + "<br>Max " + Map.MAX_MAP_HEIGHT + "</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(heightLabel, gbc);
		settingPanel.add(heightLabel);
		
		this.heightSpinner = new JSpinner(new SpinnerNumberModel(75,
				Map.MIN_MAP_HEIGHT, Map.MAX_MAP_HEIGHT, 5));
		this.heightSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.heightSpinner, gbc);
		settingPanel.add(this.heightSpinner);
		
		JLabel minRoomsLabel = new JLabel("部屋最小数");
		minRoomsLabel.setFont(m18Font);
		minRoomsLabel.setToolTipText("<html>引数名 : minRooms<br>Min " +
				Map.MIN_ROOMS + "<br>Max " + Map.MAX_ROOMS + "</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(minRoomsLabel, gbc);
		settingPanel.add(minRoomsLabel);
		
		this.minRoomsSpinner = new JSpinner(new SpinnerNumberModel(20,
				Map.MIN_ROOMS, Map.MAX_ROOMS, 1));
		this.minRoomsSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.minRoomsSpinner, gbc);
		settingPanel.add(this.minRoomsSpinner);
		
		JLabel maxRoomsLabel = new JLabel("部屋最大数");
		maxRoomsLabel.setFont(m18Font);
		maxRoomsLabel.setToolTipText("<html>引数名 : maxRooms<br>Min " +
				Map.MIN_ROOMS + "<br>Max " + Map.MAX_ROOMS +
				"<br>[部屋最小数]以上の値にする必要があります。</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(maxRoomsLabel, gbc);
		settingPanel.add(maxRoomsLabel);
		
		this.maxRoomsSpinner = new JSpinner(new SpinnerNumberModel(40,
				Map.MIN_ROOMS, Map.MAX_ROOMS, 1));
		this.maxRoomsSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.maxRoomsSpinner, gbc);
		settingPanel.add(this.maxRoomsSpinner);
		
		JLabel minRoomWidthLabel = new JLabel("部屋幅最小");
		minRoomWidthLabel.setFont(m18Font);
		minRoomWidthLabel.setToolTipText("<html>引数名 : minRoomWidth<br>Min " +
				Map.MIN_ROOM_WIDTH + "<br>Max " + Map.MAX_ROOM_WIDTH +
				"</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(minRoomWidthLabel, gbc);
		settingPanel.add(minRoomWidthLabel);
		
		this.minRoomWidthSpinner = new JSpinner(new SpinnerNumberModel(3,
				Map.MIN_ROOM_WIDTH, Map.MAX_ROOM_WIDTH, 1));
		this.minRoomWidthSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.minRoomWidthSpinner, gbc);
		settingPanel.add(this.minRoomWidthSpinner);
		
		JLabel maxRoomWidthLabel = new JLabel("部屋幅最大");
		maxRoomWidthLabel.setFont(m18Font);
		maxRoomWidthLabel.setToolTipText("<html>引数名 : maxRoomWidth<br>Min " +
				Map.MIN_ROOM_WIDTH + "<br>Max " + Map.MAX_ROOM_WIDTH +
				"<br>[部屋幅最小]以上の値にする必要があります。</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(maxRoomWidthLabel, gbc);
		settingPanel.add(maxRoomWidthLabel);
		
		this.maxRoomWidthSpinner = new JSpinner(new SpinnerNumberModel(12,
				Map.MIN_ROOM_WIDTH, Map.MAX_ROOM_WIDTH, 1));
		this.maxRoomWidthSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.maxRoomWidthSpinner, gbc);
		settingPanel.add(this.maxRoomWidthSpinner);
		
		JLabel minRoomHeightLabel = new JLabel("部屋高さ最小");
		minRoomHeightLabel.setFont(m18Font);
		minRoomHeightLabel.setToolTipText("<html>引数名 : minRoomHeight<br>Mi" +
				"n " + Map.MIN_ROOM_HEIGHT + "<br>Max " + Map.MAX_ROOM_HEIGHT +
				"</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(minRoomHeightLabel, gbc);
		settingPanel.add(minRoomHeightLabel);
		
		this.minRoomHeightSpinner = new JSpinner(new SpinnerNumberModel(3,
				Map.MIN_ROOM_HEIGHT, Map.MAX_ROOM_HEIGHT, 1));
		this.minRoomHeightSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.minRoomHeightSpinner, gbc);
		settingPanel.add(this.minRoomHeightSpinner);
		
		JLabel maxRoomHeightLabel = new JLabel("部屋高さ最大");
		maxRoomHeightLabel.setFont(m18Font);
		maxRoomHeightLabel.setToolTipText("<html>引数名 : maxRoomHeight<br>Mi" +
				"n " + Map.MIN_ROOM_HEIGHT + "<br>Max " + Map.MAX_ROOM_HEIGHT +
				"<br>[部屋高さ最小]以上の値にする必要があります。</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(maxRoomHeightLabel, gbc);
		settingPanel.add(maxRoomHeightLabel);
		
		this.maxRoomHeightSpinner = new JSpinner(new SpinnerNumberModel(12,
				Map.MIN_ROOM_HEIGHT, Map.MAX_ROOM_HEIGHT, 1));
		this.maxRoomHeightSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.maxRoomHeightSpinner, gbc);
		settingPanel.add(this.maxRoomHeightSpinner);
		
		JLabel maxConnectLengthLabel =
				new JLabel("<html>繋ぐ部屋同士<br>&nbsp;の最長距離</html>");
		maxConnectLengthLabel.setFont(m18Font);
		maxConnectLengthLabel.setToolTipText("<html>引数名 : maxConnectLength" +
				"<br>Min " + Map.MIN_MAX_CONNECT_LENGTH + "<br>Max " + Map
				.MAX_MAX_CONNECT_LENGTH + "<br>部屋と部屋を繋ぐ際の最大距離で" +
				"す。</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(maxConnectLengthLabel, gbc);
		settingPanel.add(maxConnectLengthLabel);
		
		this.maxConnectLengthSpinner =  new JSpinner(new SpinnerNumberModel(15,
				Map.MIN_MAX_CONNECT_LENGTH,
				Map.MAX_MAX_CONNECT_LENGTH, 1));
		this.maxConnectLengthSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.maxConnectLengthSpinner, gbc);
		settingPanel.add(this.maxConnectLengthSpinner);
		
		JLabel tinyRoomRatioLabel = new JLabel("1x1部屋生成率");
		tinyRoomRatioLabel.setFont(m18Font);
		tinyRoomRatioLabel.setToolTipText("<html>引数名 : tinyRoomRatio<br>Mi" +
				"n " + Map.MIN_TINY_ROOM_RATIO + "<br>Max " + Map
				.MAX_TINY_ROOM_RATIO + "</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(tinyRoomRatioLabel, gbc);
		settingPanel.add(tinyRoomRatioLabel);
		
		this.tinyRoomRatioSpinner = new JSpinner(new SpinnerNumberModel(15,
				Map.MIN_TINY_ROOM_RATIO, Map.MAX_TINY_ROOM_RATIO, 1));
		this.tinyRoomRatioSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.tinyRoomRatioSpinner, gbc);
		settingPanel.add(this.tinyRoomRatioSpinner);
		
		JLabel tinyPassageRatioLabel =
				new JLabel("<html>1x1部屋が通路<br>&nbsp;になる率</html>");
		tinyPassageRatioLabel.setFont(m18Font);
		tinyPassageRatioLabel.setToolTipText("<html>引数名 : tinyPassageRatio" +
				"<br>Min " + Map.MIN_TINY_PASSAGE_RATIO + "<br>Max " + Map
				.MAX_TINY_PASSAGE_RATIO + "</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(tinyPassageRatioLabel, gbc);
		settingPanel.add(tinyPassageRatioLabel);
		
		this.tinyPassageRatioSpinner = new JSpinner(new SpinnerNumberModel(50,
				Map.MIN_TINY_PASSAGE_RATIO, Map.MAX_TINY_PASSAGE_RATIO, 1));
		this.tinyPassageRatioSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.tinyPassageRatioSpinner, gbc);
		settingPanel.add(this.tinyPassageRatioSpinner);
		
		JLabel intersectRatioLabel = new JLabel("通路が交差する確率");
		intersectRatioLabel.setFont(m18Font);
		intersectRatioLabel.setToolTipText("<html>引数名 : intersectRatio<br>" +
				"Min " + Map.MIN_INTERSECT_RATIO + "<br>Max " + Map
				.MAX_INTERSECT_RATIO + "</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(intersectRatioLabel, gbc);
		settingPanel.add(intersectRatioLabel);
		
		this.intersectRatioSpinner = new JSpinner(new SpinnerNumberModel(50,
				Map.MIN_INTERSECT_RATIO, Map.MAX_INTERSECT_RATIO, 1));
		this.intersectRatioSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.intersectRatioSpinner, gbc);
		settingPanel.add(this.intersectRatioSpinner);
		
		JLabel minModPassagesLabel = new JLabel("通路追加最小数");
		minModPassagesLabel.setFont(m18Font);
		minModPassagesLabel.setToolTipText("<html>引数名 : modPassages<br>Min" +
				" " + Map.MIN_MOD_PASSAGES + "<br>Max " + Map.MAX_MOD_PASSAGES +
				"<br>部屋を全て生成して繋いだ後、さらに生成する通路の最小数で" +
				"す。<br>部屋の数が 1 つだけの場合は無視されます。</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(minModPassagesLabel, gbc);
		settingPanel.add(minModPassagesLabel);
		
		this.minModPassagesSpinner = new JSpinner(new SpinnerNumberModel(1,
				Map.MIN_MOD_PASSAGES, Map.MAX_MOD_PASSAGES, 1));
		this.minModPassagesSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.minModPassagesSpinner, gbc);
		settingPanel.add(this.minModPassagesSpinner);
		
		JLabel maxModPassagesLabel = new JLabel("通路追加最大数");
		maxModPassagesLabel.setFont(m18Font);
		maxModPassagesLabel.setToolTipText("<html>引数名 : modPassages<br>Min" +
				" " + Map.MIN_MOD_PASSAGES + "<br>Max " + Map.MAX_MOD_PASSAGES +
				"<br>部屋を全て生成して繋いだ後、さらに生成する通路の最大数で" +
				"す。<br>部屋の数が 1 つだけの場合は無視されます。<br>[通路追" +
				"加最小数]以上の値にする必要があります。</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(maxModPassagesLabel, gbc);
		settingPanel.add(maxModPassagesLabel);
		
		this.maxModPassagesSpinner = new JSpinner(new SpinnerNumberModel(5,
				Map.MIN_INTERSECT_RATIO, Map.MAX_INTERSECT_RATIO, 1));
		this.maxModPassagesSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.maxModPassagesSpinner, gbc);
		settingPanel.add(this.maxModPassagesSpinner);
		
		JLabel permissibleFailsLabel = new JLabel("失敗許容回数");
		permissibleFailsLabel.setFont(m18Font);
		permissibleFailsLabel.setToolTipText("<html>引数名 : permissibleFails" +
				"<br>Min " + Map.MIN_PERMISSIBLE_FAILS + "<br>Max " + Map
				.MAX_PERMISSIBLE_FAILS + "<br>部屋が重なる失敗と、通路の生成" +
				"の失敗を許容する回数です。<br>失敗数がこの値に届いた時点でマ" +
				"ップの生成は中止しますが、<br>最低でも 1 つの部屋が生成され" +
				"ることが保証されます。<br>この値が大きいと、生成に時間がかか" +
				"る可能性があります。</html>");
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(permissibleFailsLabel, gbc);
		settingPanel.add(permissibleFailsLabel);
		
		this.permissibleFailsSpinner = new JSpinner(new SpinnerNumberModel(2500,
				Map.MIN_PERMISSIBLE_FAILS, Map.MAX_PERMISSIBLE_FAILS, 100));
		this.permissibleFailsSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.permissibleFailsSpinner, gbc);
		settingPanel.add(this.permissibleFailsSpinner);
		
		this.autoSeedsCheckBox = new JCheckBox("乱数の種を自動設定", true);
		this.autoSeedsCheckBox.addChangeListener(new ChangeListener() {
			@Override
            public final void stateChanged(ChangeEvent e) {
				boolean auto = !GraphicalFrame.this.autoSeedsCheckBox
						.isSelected();
				GraphicalFrame.this.seedXSpinner.setEnabled(auto);
				GraphicalFrame.this.seedYSpinner.setEnabled(auto);
				GraphicalFrame.this.seedZSpinner.setEnabled(auto);
				GraphicalFrame.this.seedWSpinner.setEnabled(auto);
			}
		});
		this.autoSeedsCheckBox.setFont(m18Font);
		gbc.anchor = GridBagConstraints.WEST;
		++gbc.gridwidth;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(this.autoSeedsCheckBox, gbc);
		settingPanel.add(this.autoSeedsCheckBox);
		
		long uIntMax = 0xffffffffL;
		
		String seedToolTip = "<html>Min " + 0 + "<br>Max " + uIntMax + "<br>" +
				"乱数の種です。X, Y, Z, W の値と生成条件が同じであれば、<br>" +
				"同じ形のマップが生成されます。<br>4 つ全ての値が 0 の場合、" +
				"値が自動生成されます。</html>";
		
		JLabel seedXLabel = new JLabel("Seed X");
		seedXLabel.setFont(m18Font);
		seedXLabel.setToolTipText(seedToolTip);
		//gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridwidth;
		//gbc.gridx = 0;
		++gbc.gridy;
		gbl.setConstraints(seedXLabel, gbc);
		settingPanel.add(seedXLabel);
		
		this.seedXSpinner = new JSpinner(new SpinnerNumberModel(Long
				.valueOf(0L), Long.valueOf(0L), Long.valueOf(uIntMax),
				Long.valueOf(1L)));
		this.seedXSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.seedXSpinner, gbc);
		settingPanel.add(this.seedXSpinner);
		
		JLabel seedYLabel = new JLabel("Seed Y");
		seedYLabel.setFont(m18Font);
		seedYLabel.setToolTipText(seedToolTip);
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(seedYLabel, gbc);
		settingPanel.add(seedYLabel);
		
		this.seedYSpinner = new JSpinner(new SpinnerNumberModel(Long
				.valueOf(0L), Long.valueOf(0L), Long.valueOf(uIntMax),
				Long.valueOf(1L)));
		this.seedYSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.seedYSpinner, gbc);
		settingPanel.add(this.seedYSpinner);
		
		JLabel seedZLabel = new JLabel("Seed Z");
		seedZLabel.setFont(m18Font);
		seedZLabel.setToolTipText(seedToolTip);
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(seedZLabel, gbc);
		settingPanel.add(seedZLabel);
		
		this.seedZSpinner = new JSpinner(new SpinnerNumberModel(Long
				.valueOf(0L), Long.valueOf(0L), Long.valueOf(uIntMax),
				Long.valueOf(1L)));
		this.seedZSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.seedZSpinner, gbc);
		settingPanel.add(this.seedZSpinner);
		
		JLabel seedWLabel = new JLabel("Seed W");
		seedWLabel.setFont(m18Font);
		seedWLabel.setToolTipText(seedToolTip);
		gbc.anchor = GridBagConstraints.WEST;
		--gbc.gridx;
		++gbc.gridy;
		gbl.setConstraints(seedWLabel, gbc);
		settingPanel.add(seedWLabel);
		
		this.seedWSpinner = new JSpinner(new SpinnerNumberModel(Long
				.valueOf(0L), Long.valueOf(0L), Long.valueOf(uIntMax),
				Long.valueOf(1L)));
		this.seedWSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		++gbc.gridx;
		gbl.setConstraints(this.seedWSpinner, gbc);
		settingPanel.add(this.seedWSpinner);
		
		// 右側中央
		JPanel rightMiddlePanel = new JPanel(new BorderLayout());
		rightPanel2.add(rightMiddlePanel, BorderLayout.CENTER);
		
		gbl = new GridBagLayout();
		
		JPanel visualSettingPanel = new JPanel(gbl);
		tb = new TitledBorder(new EtchedBorder(), "描画設定");
		tb.setTitleFont(m15Font);
		visualSettingPanel.setBorder(tb);
		rightMiddlePanel.add(visualSettingPanel, BorderLayout.NORTH);
		
		this.animationCheckBox = new JCheckBox("アニメーション");
		this.animationCheckBox.addChangeListener(new ChangeListener() {
			@Override
            public final void stateChanged(ChangeEvent e) {
				boolean enabled = GraphicalFrame.this.animationCheckBox
						.isSelected();
                GraphicalFrame.this.drawRoomSpeedLabel.setEnabled(enabled);
                GraphicalFrame.this.drawRoomSpeedSpinner.setEnabled(enabled);
                GraphicalFrame.this.fillRoomCheckBox.setEnabled(enabled);
                GraphicalFrame.this.drawPassageSpeedLabel.setEnabled(enabled);
                GraphicalFrame.this.drawPassageSpeedSpinner.setEnabled(enabled);
            }
        });
		this.animationCheckBox.setFont(m18Font);
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 6;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbl.setConstraints(this.animationCheckBox, gbc);
		visualSettingPanel.add(this.animationCheckBox);
		
		String drawToolTip = "(Min 1, Max 100)";
		
		this.drawRoomSpeedLabel = new JLabel("部屋描画速度");
		this.drawRoomSpeedLabel.setFont(m18Font);
		this.drawRoomSpeedLabel.setToolTipText(drawToolTip);
		//gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 5;
		gbc.gridx = 0;
		++gbc.gridy;
		gbl.setConstraints(this.drawRoomSpeedLabel, gbc);
		visualSettingPanel.add(this.drawRoomSpeedLabel);
		
		this.drawRoomSpeedSpinner =
				new JSpinner(new SpinnerNumberModel(4, 1, 100, 1));
		this.drawRoomSpeedSpinner.addChangeListener(new ChangeListener() {
			@Override
			public final void stateChanged(ChangeEvent e) {
				GraphicalFrame.this.graphicalPanel.setRoomDrawSpeed((int)
						GraphicalFrame.this.drawRoomSpeedSpinner.getValue());
			}
		});
		this.drawRoomSpeedSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridwidth = 1;
		gbc.gridx = 5;
		gbl.setConstraints(this.drawRoomSpeedSpinner, gbc);
		visualSettingPanel.add(this.drawRoomSpeedSpinner);
		
		this.fillRoomCheckBox = new JCheckBox("部屋を一度に塗りつぶす");
		this.fillRoomCheckBox.addChangeListener(new ChangeListener() {
			@Override
            public final void stateChanged(ChangeEvent e) {
                GraphicalFrame.this.graphicalPanel.setFillRoom(GraphicalFrame
                		.this.fillRoomCheckBox.isSelected());
            }
        });
		this.fillRoomCheckBox.setFont(m18Font);
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 6;
		gbc.gridx = 0;
		++gbc.gridy;
		gbl.setConstraints(this.fillRoomCheckBox, gbc);
		visualSettingPanel.add(this.fillRoomCheckBox);
		
		this.drawPassageSpeedLabel = new JLabel("通路描画速度");
		this.drawPassageSpeedLabel.setFont(m18Font);
		this.drawPassageSpeedLabel.setToolTipText(drawToolTip);
		//gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 5;
		gbc.gridx = 0;
		++gbc.gridy;
		gbl.setConstraints(this.drawPassageSpeedLabel, gbc);
		visualSettingPanel.add(this.drawPassageSpeedLabel);
		
		this.drawPassageSpeedSpinner =
				new JSpinner(new SpinnerNumberModel(20, 1, 100, 1));
		this.drawPassageSpeedSpinner.addChangeListener(new ChangeListener() {
			@Override
			public final void stateChanged(ChangeEvent e) {
				GraphicalFrame.this.graphicalPanel.setPassageDrawSpeed((int)
						GraphicalFrame.this.drawPassageSpeedSpinner.getValue());
			}
		});
		this.drawPassageSpeedSpinner.setFont(m15Font);
		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridwidth = 1;
		gbc.gridx = 5;
		gbl.setConstraints(this.drawPassageSpeedSpinner, gbc);
		visualSettingPanel.add(this.drawPassageSpeedSpinner);
		
		ChangeListener cl;
		
		JLabel roomColorLabel = new JLabel("部屋の色");
		roomColorLabel.setFont(m18Font);
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 6;
		gbc.gridx = 0;
		++gbc.gridy;
		gbl.setConstraints(roomColorLabel, gbc);
		visualSettingPanel.add(roomColorLabel);
		
		cl = new ChangeListener() {
			@Override
			public final void stateChanged(ChangeEvent e) {
				GraphicalFrame.this.changedRoomColor = true;
			}
		};
		
		JLabel roomRedLabel = new JLabel(" Red ");
		roomRedLabel.setFont(m18Font);
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.gridwidth = 1;
		++gbc.gridy;
		gbl.setConstraints(roomRedLabel, gbc);
		visualSettingPanel.add(roomRedLabel);
		
		this.roomRedSpinner =
				new JSpinner(new SpinnerNumberModel(255, 0, 255, 16));
		this.roomRedSpinner.addChangeListener(cl);
		this.roomRedSpinner.setFont(m15Font);
		++gbc.gridx;
		gbl.setConstraints(this.roomRedSpinner, gbc);
		visualSettingPanel.add(this.roomRedSpinner);
		
		JLabel roomGreenLabel = new JLabel(" Green ");
		roomGreenLabel.setFont(m18Font);
		++gbc.gridx;
		gbl.setConstraints(roomGreenLabel, gbc);
		visualSettingPanel.add(roomGreenLabel);
		
		this.roomGreenSpinner =
				new JSpinner(new SpinnerNumberModel(255, 0, 255, 16));
		this.roomGreenSpinner.addChangeListener(cl);
		this.roomGreenSpinner.setFont(m15Font);
		++gbc.gridx;
		gbl.setConstraints(this.roomGreenSpinner, gbc);
		visualSettingPanel.add(this.roomGreenSpinner);
		
		JLabel roomBlueLabel = new JLabel(" Blue ");
		roomBlueLabel.setFont(m18Font);
		++gbc.gridx;
		gbl.setConstraints(roomBlueLabel, gbc);
		visualSettingPanel.add(roomBlueLabel);
		
		this.roomBlueSpinner =
				new JSpinner(new SpinnerNumberModel(255, 0, 255, 16));
		this.roomBlueSpinner.addChangeListener(cl);
		this.roomBlueSpinner.setFont(m15Font);
		++gbc.gridx;
		gbl.setConstraints(this.roomBlueSpinner, gbc);
		visualSettingPanel.add(this.roomBlueSpinner);
		
		JLabel passageColorLabel = new JLabel("通路の色");
		passageColorLabel.setFont(m18Font);
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 6;
		gbc.gridx = 0;
		++gbc.gridy;
		gbl.setConstraints(passageColorLabel, gbc);
		visualSettingPanel.add(passageColorLabel);
		
		cl = new ChangeListener() {
			@Override
			public final void stateChanged(ChangeEvent e) {
				GraphicalFrame.this.changedPassageColor = true;
			}
		};
		
		JLabel passageRedLabel = new JLabel(" Red ");
		passageRedLabel.setFont(m18Font);
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.gridwidth = 1;
		++gbc.gridy;
		gbl.setConstraints(passageRedLabel, gbc);
		visualSettingPanel.add(passageRedLabel);
		
		this.passageRedSpinner =
				new JSpinner(new SpinnerNumberModel(128, 0, 255, 16));
		this.passageRedSpinner.addChangeListener(cl);
		this.passageRedSpinner.setFont(m15Font);
		++gbc.gridx;
		gbl.setConstraints(this.passageRedSpinner, gbc);
		visualSettingPanel.add(this.passageRedSpinner);
		
		JLabel passageGreenLabel = new JLabel(" Green ");
		passageGreenLabel.setFont(m18Font);
		++gbc.gridx;
		gbl.setConstraints(passageGreenLabel, gbc);
		visualSettingPanel.add(passageGreenLabel);
		
		this.passageGreenSpinner =
				new JSpinner(new SpinnerNumberModel(128, 0, 255, 16));
		this.passageGreenSpinner.addChangeListener(cl);
		this.passageGreenSpinner.setFont(m15Font);
		++gbc.gridx;
		gbl.setConstraints(this.passageGreenSpinner, gbc);
		visualSettingPanel.add(this.passageGreenSpinner);
		
		JLabel passageBlueLabel = new JLabel(" Blue ");
		passageBlueLabel.setFont(m18Font);
		++gbc.gridx;
		gbl.setConstraints(passageBlueLabel, gbc);
		visualSettingPanel.add(passageBlueLabel);
		
		this.passageBlueSpinner =
				new JSpinner(new SpinnerNumberModel(128, 0, 255, 16));
		this.passageBlueSpinner.addChangeListener(cl);
		this.passageBlueSpinner.setFont(m15Font);
		++gbc.gridx;
		gbl.setConstraints(this.passageBlueSpinner, gbc);
		visualSettingPanel.add(this.passageBlueSpinner);
		
		JLabel doorwayColorLabel = new JLabel("部屋と通路の接点の色");
		doorwayColorLabel.setFont(m18Font);
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 6;
		gbc.gridx = 0;
		++gbc.gridy;
		gbl.setConstraints(doorwayColorLabel, gbc);
		visualSettingPanel.add(doorwayColorLabel);
		
		cl = new ChangeListener() {
			@Override
			public final void stateChanged(ChangeEvent e) {
				GraphicalFrame.this.changedDoorwayColor = true;
			}
		};
		
		JLabel doorwayRedLabel = new JLabel(" Red ");
		doorwayRedLabel.setFont(m18Font);
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.gridwidth = 1;
		++gbc.gridy;
		gbl.setConstraints(doorwayRedLabel, gbc);
		visualSettingPanel.add(doorwayRedLabel);
		
		this.doorwayRedSpinner =
				new JSpinner(new SpinnerNumberModel(255, 0, 255, 16));
		this.doorwayRedSpinner.addChangeListener(cl);
		this.doorwayRedSpinner.setFont(m15Font);
		++gbc.gridx;
		gbl.setConstraints(this.doorwayRedSpinner, gbc);
		visualSettingPanel.add(this.doorwayRedSpinner);
		
		JLabel doorwayGreenLabel = new JLabel(" Green ");
		doorwayGreenLabel.setFont(m18Font);
		++gbc.gridx;
		gbl.setConstraints(doorwayGreenLabel, gbc);
		visualSettingPanel.add(doorwayGreenLabel);
		
		this.doorwayGreenSpinner =
				new JSpinner(new SpinnerNumberModel(0, 0, 255, 16));
		this.doorwayGreenSpinner.addChangeListener(cl);
		this.doorwayGreenSpinner.setFont(m15Font);
		++gbc.gridx;
		gbl.setConstraints(this.doorwayGreenSpinner, gbc);
		visualSettingPanel.add(this.doorwayGreenSpinner);
		
		JLabel doorwayBlueLabel = new JLabel(" Blue ");
		doorwayBlueLabel.setFont(m18Font);
		++gbc.gridx;
		gbl.setConstraints(doorwayBlueLabel, gbc);
		visualSettingPanel.add(doorwayBlueLabel);
		
		this.doorwayBlueSpinner =
				new JSpinner(new SpinnerNumberModel(0, 0, 255, 16));
		this.doorwayBlueSpinner.addChangeListener(cl);
		this.doorwayBlueSpinner.setFont(m15Font);
		++gbc.gridx;
		gbl.setConstraints(this.doorwayBlueSpinner, gbc);
		visualSettingPanel.add(this.doorwayBlueSpinner);
		
		this.pack();
		
		this.imageChooser = new JFileChooser();
		this.imageChooser.setFileFilter(new FileNameExtensionFilter("Pngファ" +
				"イル", "png"));
		
		ToolTipManager.sharedInstance().setDismissDelay(180000);

		this.autoSeedsCheckBox.getChangeListeners()[0].stateChanged(null);
		this.animationCheckBox.getChangeListeners()[0].stateChanged(null);
		
		this.coordinateLabel.setPreferredSize(this.coordinateLabel.getPreferredSize());
		this.hideCoordinates();
		
	}

	public static final void main(String args[]) {
		EventQueue.invokeLater(new Runnable() {
			@Override
			public final void run() {
				try {
					DisplayMode dm = GraphicsEnvironment
							.getLocalGraphicsEnvironment()
							.getDefaultScreenDevice().getDisplayMode();
					int w = dm.getWidth(), h = dm.getHeight();
					GraphicalFrame frame = new GraphicalFrame();
					Dimension size = new Dimension(w < 800 ?
							(w >> 2) * 3 : w >= 1024 ? 800 : 640, h < 600 ?
									(h >> 2) * 3 : h >= 768 ? 600 : 480);
					frame.setSize(size);
					frame.setMinimumSize(size);
					frame.setVisible(true);
					frame.setLocationRelativeTo(null);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	@Override
	public final void mouseEntered(MouseEvent e){ }

	@Override
	public final void mouseExited(MouseEvent e){
		this.hideCoordinates();
	}

	@Override
	public final void mousePressed(MouseEvent e){ }

	@Override
	public final void mouseReleased(MouseEvent e){ }

	@Override
	public final void mouseClicked(MouseEvent e){ }
	
	@Override
	public final void mouseDragged(MouseEvent e) { }
	
	@Override
	public final void mouseMoved(MouseEvent e) {
		this.showCoordinates(e.getX() >> 2, e.getY() >> 2);
	}
	
	private final void showMessage(String text) {
		this.messageLabel.setText(text);
	}
	
	private final void hideMessage() {
		this.messageLabel.setText("");
	}
	
	private final void showCoordinates(int x, int y) {
		this.coordinateLabel.setText(String.format("X(%1$4d), Y(%2$4d)", x, y));
	}
	
	private final void hideCoordinates() {
		this.coordinateLabel.setText("");
	}
    
    private final void changeRoomColor() {
    	if (this.changedRoomColor) {
    		GraphicalMap.setRoomColor((int)this.roomRedSpinner.getValue(),
    				(int)this.roomGreenSpinner.getValue(),
    				(int)this.roomBlueSpinner.getValue());
    		this.changedRoomColor = false;
    	}
    }
    
    private final void changePassageColor() {
    	if (this.changedPassageColor) {
    		GraphicalMap.setPassageColor((int)this.passageRedSpinner.getValue(),
    				(int)this.passageGreenSpinner.getValue(),
    				(int)this.passageBlueSpinner.getValue());
    		this.changedPassageColor = false;
    	}
    }
    
    private final void changeDoorwayColor() {
    	if (this.changedDoorwayColor) {
    		GraphicalMap.setDoorwayColor((int)this.doorwayRedSpinner.getValue(),
    				(int)this.doorwayGreenSpinner.getValue(),
    				(int)this.doorwayBlueSpinner.getValue());
    		this.changedDoorwayColor = false;
    	}
    }

	private final void generate() {
		this.hideMessage();
		this.changeRoomColor();
		this.changePassageColor();
		this.changeDoorwayColor();
		int x, y, z, w;
		if (this.autoSeedsCheckBox.isSelected()) {
			x = 0;
			y = 0;
			z = 0;
			w = 0;
		} else {
			x = (int)(long)this.seedXSpinner.getValue();
			y = (int)(long)this.seedYSpinner.getValue();
			z = (int)(long)this.seedZSpinner.getValue();
			w = (int)(long)this.seedWSpinner.getValue();
		}
		int minModPassages = (int)this.minModPassagesSpinner.getValue();
		int maxModPassages = (int)this.maxModPassagesSpinner.getValue();
		int modPassages = maxModPassages > minModPassages ?
				RND.next(minModPassages, maxModPassages + 1) : minModPassages;
		try {
			this.graphicalPanel.generate(this.animationCheckBox.isSelected(),
					x, y, z, w, (int)this.widthSpinner.getValue(),
					(int)this.heightSpinner.getValue(),
					(int)this.minRoomsSpinner.getValue(),
					(int)this.maxRoomsSpinner.getValue(),
					(int)this.minRoomWidthSpinner.getValue(),
					(int)this.maxRoomWidthSpinner.getValue(),
					(int)this.minRoomHeightSpinner.getValue(),
					(int)this.maxRoomHeightSpinner.getValue(),
					(int)this.maxConnectLengthSpinner.getValue(),
					(int)this.tinyRoomRatioSpinner.getValue(),
					(int)this.tinyPassageRatioSpinner.getValue(),
					(int)this.intersectRatioSpinner.getValue(), modPassages,
					(int)this.permissibleFailsSpinner.getValue());
		} catch (IllegalArgumentException e) {
			this.outputTextArea.setText(e.getMessage());
			this.graphicalPanel.destroy();
			this.generateButton.setEnabled(true);
		}
	}
	
	private final void showGeneratedMapSetting(Map map) {
		SB.append("幅 = ")
				.append(String.format("%1$4d", map.getMapWidth()))
				.append("\n高さ = ")
				.append(String.format("%1$4d", map.getMapHeight()))
				.append("\n部屋数 = ")
				.append(String.format("%1$4d", map.getMinRooms()))
				.append(" ~ ")
				.append(String.format("%1$4d", map.getMaxRooms()))
				.append("\n部屋幅 = ")
				.append(String.format("%1$2d", map.getMinRoomWidth()))
				.append(" ~ ")
				.append(String.format("%1$2d", map.getMaxRoomWidth()))
				.append("\n部屋高さ = ")
				.append(String.format("%1$2d", map.getMinRoomHeight()))
				.append(" ~ ")
				.append(String.format("%1$2d", map.getMaxRoomHeight()))
				.append("\n繋ぐ部屋同士の最長距離 = ")
				.append(String.format("%1$3d",
						map.getMaxConnectLength()))
				.append("\n1x1の部屋生成率 = ")
				.append(String.format("%1$3d", map.getTinyRoomRatio()))
				.append(" %\n1x1の部屋を通路にする確率 = ")
				.append(String.format("%1$3d", map.getTinyPassageRatio()))
				.append(" %\n通路交差率  = ")
				.append(String.format("%1$3d", map.getIntersectRatio()))
				.append(" %\n通路追加数  = ")
				.append(String.format("%1$2d", map.getModPassages()))
				.append("\n失敗許容回数 = ")
				.append(String.format("%1$7d", map.getPermissibleFails()))
				.append("\n失敗回数 = ")
				.append(String.format("%1$7d", map.getFails()))
				.append("\nSeed X = ")
				.append(String.format("%1$10d", map.getSeedX()))
				.append("\nSeed Y = ")
				.append(String.format("%1$10d", map.getSeedY()))
				.append("\nSeed Z = ")
				.append(String.format("%1$10d", map.getSeedZ()))
				.append("\nSeed W = ")
				.append(String.format("%1$10d", map.getSeedW()));
		this.outputTextArea.setText(SB.toString());
		SB.replace(0, SB.length(), "");
		// SB 生成時の引数にする
		//System.out.println(this.outputTextArea.getText().length());
	}
	
	private final void saveImage() {
		if (this.imageChooser.showSaveDialog(this) != JFileChooser
				.APPROVE_OPTION) {
			return;
		}
		if (this.imageChooser.getSelectedFile().exists()) {
			if (JOptionPane.showConfirmDialog(this.contentPane, "ファイル " +
					this.imageChooser.getSelectedFile().getName() +
					" は既に存在しますが上書きしてもよいですか？", "確認",
					JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION) {
				return;
			}
		}
		try {
			ImageIO.write(this.graphicalPanel.getBufferedImage(), "png",
					this.imageChooser.getSelectedFile());
			this.showMessage("マップ画像を保存しました。");
		} catch (IOException e) {
			e.printStackTrace();
			this.showMessage("マップ画像を保存できませんでした。");
		}
	}
	
	final void drew(Map map) {
		this.destroyButton.setEnabled(true);
		this.regenerateButton.setEnabled(true);
		this.saveImageButton.setEnabled(true);
		this.showGeneratedMapSetting(map);
		GraphicalMap.clearGraphicalGrids();
	}

}
